@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -30,12 +30,6 @@
 
 #include "gmime-filter-gzip.h"
 
-#ifdef ENABLE_WARNINGS
-#define w(x) x
-#else
-#define w(x)
-#endif /* ENABLE_WARNINGS */
-
 
 /**
  * SECTION: gmime-filter-gzip
@@ -237,7 +231,7 @@ gzip_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 	do {
 		/* FIXME: handle error cases? */
 		if ((retval = deflate (priv->stream, flush)) != Z_OK)
-			w(fprintf (stderr, "gzip: %d: %s\n", retval, priv->stream->msg));
+			fprintf (stderr, "gzip: %d: %s\n", retval, priv->stream->msg);
 		
 		if (flush == Z_FULL_FLUSH) {
 			size_t outlen;
@@ -390,10 +384,8 @@ gunzip_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
 	
 	do {
 		/* FIXME: handle error cases? */
-		/* Note: Z_BUF_ERROR is not really an error unless there is input available */
-		if ((retval = inflate (priv->stream, flush)) != Z_OK &&
-		    !(retval == Z_BUF_ERROR && !priv->stream->avail_in))
-			w(fprintf (stderr, "gunzip: %d: %s\n", retval, priv->stream->msg));
+		if ((retval = inflate (priv->stream, flush)) != Z_OK)
+			fprintf (stderr, "gunzip: %d: %s\n", retval, priv->stream->msg);
 		
 		if (flush == Z_FULL_FLUSH) {
 			size_t outlen;
