@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -29,6 +29,7 @@
 #include "gmime-common.h"
 #include "gmime-object.h"
 #include "gmime-stream-mem.h"
+#include "gmime-events.h"
 #include "gmime-utils.h"
 
 
@@ -38,7 +39,7 @@
  * @short_description: Abstract MIME objects
  * @see_also:
  *
- * #GMimeObject is an abstract class from which all Message and Mime
+ * #GMimeObject is an abstract class from which all message and MIME
  * parts are derived.
  **/
 
@@ -61,20 +62,21 @@ static void g_mime_object_class_init (GMimeObjectClass *klass);
 static void g_mime_object_init (GMimeObject *object, GMimeObjectClass *klass);
 static void g_mime_object_finalize (GObject *object);
 
-static void prepend_header (GMimeObject *object, const char *name, const char *value);
-static void append_header (GMimeObject *object, const char *name, const char *value);
-static void set_header (GMimeObject *object, const char *name, const char *value);
-static const char *get_header (GMimeObject *object, const char *name);
-static gboolean remove_header (GMimeObject *object, const char *name);
-static void set_content_type (GMimeObject *object, GMimeContentType *content_type);
-static char *get_headers (GMimeObject *object);
-static ssize_t write_to_stream (GMimeObject *object, GMimeStream *stream);
+static void object_prepend_header (GMimeObject *object, const char *name, const char *value);
+static void object_append_header (GMimeObject *object, const char *name, const char *value);
+static void object_set_header (GMimeObject *object, const char *name, const char *value);
+static const char *object_get_header (GMimeObject *object, const char *name);
+static gboolean object_remove_header (GMimeObject *object, const char *name);
+static void object_set_content_type (GMimeObject *object, GMimeContentType *content_type);
+static char *object_get_headers (GMimeObject *object);
+static ssize_t object_write_to_stream (GMimeObject *object, GMimeStream *stream);
+static void object_encode (GMimeObject *object, GMimeEncodingConstraint constraint);
 
 static ssize_t write_content_type (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_disposition (GMimeStream *stream, const char *name, const char *value);
 
-static void content_type_changed (GMimeContentType *content_type, GMimeObject *object);
-static void content_disposition_changed (GMimeContentDisposition *disposition, GMimeObject *object);
+static void content_type_changed (GMimeContentType *content_type, gpointer args, GMimeObject *object);
+static void content_disposition_changed (GMimeContentDisposition *disposition, gpointer args, GMimeObject *object);
 
 
 static GHashTable *type_hash = NULL;
@@ -117,14 +119,15 @@ g_mime_object_class_init (GMimeObjectClass *klass)
 	
 	object_class->finalize = g_mime_object_finalize;
 	
-	klass->prepend_header = prepend_header;
-	klass->append_header = append_header;
-	klass->remove_header = remove_header;
-	klass->set_header = set_header;
-	klass->get_header = get_header;
-	klass->set_content_type = set_content_type;
-	klass->get_headers = get_headers;
-	klass->write_to_stream = write_to_stream;
+	klass->prepend_header = object_prepend_header;
+	klass->append_header = object_append_header;
+	klass->remove_header = object_remove_header;
+	klass->set_header = object_set_header;
+	klass->get_header = object_get_header;
+	klass->set_content_type = object_set_content_type;
+	klass->get_headers = object_get_headers;
+	klass->write_to_stream = object_write_to_stream;
+	klass->encode = object_encode;
 }
 
 static void
@@ -145,16 +148,12 @@ g_mime_object_finalize (GObject *object)
 	GMimeObject *mime = (GMimeObject *) object;
 	
 	if (mime->content_type) {
-		g_signal_handlers_disconnect_matched (mime->content_type,
-						      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
-						      0, 0, NULL, content_type_changed, object);
+		g_mime_event_remove (mime->content_type->priv, (GMimeEventCallback) content_type_changed, object);
 		g_object_unref (mime->content_type);
 	}
 	
 	if (mime->disposition) {
-		g_signal_handlers_disconnect_matched (mime->disposition,
-						      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
-						      0, 0, NULL, content_disposition_changed, object);
+		g_mime_event_remove (mime->disposition->priv, (GMimeEventCallback) content_disposition_changed, object);
 		g_object_unref (mime->disposition);
 	}
 	
@@ -194,7 +193,7 @@ write_content_type (GMimeStream *stream, const char *name, const char *value)
 }
 
 static void
-content_type_changed (GMimeContentType *content_type, GMimeObject *object)
+content_type_changed (GMimeContentType *content_type, gpointer args, GMimeObject *object)
 {
 	GMimeParam *params;
 	GString *string;
@@ -240,7 +239,7 @@ write_disposition (GMimeStream *stream, const char *name, const char *value)
 }
 
 static void
-content_disposition_changed (GMimeContentDisposition *disposition, GMimeObject *object)
+content_disposition_changed (GMimeContentDisposition *disposition, gpointer args, GMimeObject *object)
 {
 	char *str;
 	
@@ -395,16 +394,14 @@ g_mime_object_new_type (const char *type, const char *subtype)
 
 
 static void
-set_content_type (GMimeObject *object, GMimeContentType *content_type)
+object_set_content_type (GMimeObject *object, GMimeContentType *content_type)
 {
 	if (object->content_type) {
-		g_signal_handlers_disconnect_matched (object->content_type,
-						      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
-						      0, 0, NULL, content_type_changed, object);
+		g_mime_event_remove (object->content_type->priv, (GMimeEventCallback) content_type_changed, object);
 		g_object_unref (object->content_type);
 	}
 	
-	g_signal_connect (content_type, "changed", G_CALLBACK (content_type_changed), object);
+	g_mime_event_add (content_type->priv, (GMimeEventCallback) content_type_changed, object);
 	object->content_type = content_type;
 	g_object_ref (content_type);
 }
@@ -446,7 +443,7 @@ g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *content_t
 	
 	GMIME_OBJECT_GET_CLASS (object)->set_content_type (object, content_type);
 	
-	content_type_changed (content_type, object);
+	content_type_changed (content_type, NULL, object);
 }
 
 
@@ -538,13 +535,11 @@ static void
 _g_mime_object_set_content_disposition (GMimeObject *object, GMimeContentDisposition *disposition)
 {
 	if (object->disposition) {
-		g_signal_handlers_disconnect_matched (object->disposition,
-						      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
-						      0, 0, NULL, content_disposition_changed, object);
+		g_mime_event_remove (object->disposition->priv, (GMimeEventCallback) content_disposition_changed, object);
 		g_object_unref (object->disposition);
 	}
 	
-	g_signal_connect (disposition, "changed", G_CALLBACK (content_disposition_changed), object);
+	g_mime_event_add (disposition->priv, (GMimeEventCallback) content_disposition_changed, object);
 	object->disposition = disposition;
 	g_object_ref (disposition);
 }
@@ -569,7 +564,7 @@ g_mime_object_set_content_disposition (GMimeObject *object, GMimeContentDisposit
 	
 	_g_mime_object_set_content_disposition (object, disposition);
 	
-	content_disposition_changed (disposition, object);
+	content_disposition_changed (disposition, NULL, object);
 }
 
 
@@ -733,8 +728,11 @@ process_header (GMimeObject *object, const char *header, const char *value)
 	GMimeContentType *content_type;
 	guint i;
 	
+	if (g_ascii_strncasecmp (header, "Content-", 8) != 0)
+		return FALSE;
+	
 	for (i = 0; i < G_N_ELEMENTS (content_headers); i++) {
-		if (!g_ascii_strcasecmp (content_headers[i], header))
+		if (!g_ascii_strcasecmp (content_headers[i] + 8, header + 8))
 			break;
 	}
 	
@@ -763,7 +761,7 @@ process_header (GMimeObject *object, const char *header, const char *value)
 }
 
 static void
-prepend_header (GMimeObject *object, const char *header, const char *value)
+object_prepend_header (GMimeObject *object, const char *header, const char *value)
 {
 	if (!process_header (object, header, value))
 		g_mime_header_list_prepend (object->headers, header, value);
@@ -776,7 +774,7 @@ prepend_header (GMimeObject *object, const char *header, const char *value)
  * @header: header name
  * @value: header value
  *
- * Prepends a raw, unprocessed header to the MIME object.
+ * Prepends a header to the MIME object.
  **/
 void
 g_mime_object_prepend_header (GMimeObject *object, const char *header, const char *value)
@@ -789,7 +787,7 @@ g_mime_object_prepend_header (GMimeObject *object, const char *header, const cha
 }
 
 static void
-append_header (GMimeObject *object, const char *header, const char *value)
+object_append_header (GMimeObject *object, const char *header, const char *value)
 {
 	if (!process_header (object, header, value))
 		g_mime_header_list_append (object->headers, header, value);
@@ -802,7 +800,7 @@ append_header (GMimeObject *object, const char *header, const char *value)
  * @header: header name
  * @value: header value
  *
- * Appends a raw, unprocessed header to the MIME object.
+ * Appends a header to the MIME object.
  **/
 void
 g_mime_object_append_header (GMimeObject *object, const char *header, const char *value)
@@ -816,7 +814,7 @@ g_mime_object_append_header (GMimeObject *object, const char *header, const char
 
 
 static void
-set_header (GMimeObject *object, const char *header, const char *value)
+object_set_header (GMimeObject *object, const char *header, const char *value)
 {
 	if (!process_header (object, header, value))
 		g_mime_header_list_set (object->headers, header, value);
@@ -829,7 +827,7 @@ set_header (GMimeObject *object, const char *header, const char *value)
  * @header: header name
  * @value: header value
  *
- * Sets an arbitrary raw, unprocessed header on the MIME object.
+ * Sets an arbitrary header on the MIME object.
  **/
 void
 g_mime_object_set_header (GMimeObject *object, const char *header, const char *value)
@@ -843,7 +841,7 @@ g_mime_object_set_header (GMimeObject *object, const char *header, const char *v
 
 
 static const char *
-get_header (GMimeObject *object, const char *header)
+object_get_header (GMimeObject *object, const char *header)
 {
 	return g_mime_header_list_get (object->headers, header);
 }
@@ -854,10 +852,11 @@ get_header (GMimeObject *object, const char *header)
  * @object: a #GMimeObject
  * @header: header name
  *
- * Gets the raw, unprocessed value of the requested header.
+ * Gets the value of the requested header if it exists or %NULL
+ * otherwise.
  *
- * Returns: the raw, unprocessed value of the requested header if it
- * exists or %NULL otherwise.
+ * Returns: the value of the header @header if it exists or %NULL
+ * otherwise.
  **/
 const char *
 g_mime_object_get_header (GMimeObject *object, const char *header)
@@ -870,7 +869,7 @@ g_mime_object_get_header (GMimeObject *object, const char *header)
 
 
 static gboolean
-remove_header (GMimeObject *object, const char *header)
+object_remove_header (GMimeObject *object, const char *header)
 {
 	guint i;
 	
@@ -882,9 +881,7 @@ remove_header (GMimeObject *object, const char *header)
 	switch (i) {
 	case HEADER_CONTENT_DISPOSITION:
 		if (object->disposition) {
-			g_signal_handlers_disconnect_matched (object->disposition,
-							      G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
-							      0, 0, NULL, content_disposition_changed, object);
+			g_mime_event_remove (object->disposition->priv, (GMimeEventCallback) content_disposition_changed, object);
 			g_object_unref (object->disposition);
 			object->disposition = NULL;
 		}
@@ -925,7 +922,7 @@ g_mime_object_remove_header (GMimeObject *object, const char *header)
 
 
 static char *
-get_headers (GMimeObject *object)
+object_get_headers (GMimeObject *object)
 {
 	return g_mime_header_list_to_string (object->headers);
 }
@@ -950,7 +947,7 @@ g_mime_object_get_headers (GMimeObject *object)
 
 
 static ssize_t
-write_to_stream (GMimeObject *object, GMimeStream *stream)
+object_write_to_stream (GMimeObject *object, GMimeStream *stream)
 {
 	return -1;
 }
@@ -975,6 +972,31 @@ g_mime_object_write_to_stream (GMimeObject *object, GMimeStream *stream)
 }
 
 
+static void
+object_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
+{
+	return;
+}
+
+
+/**
+ * g_mime_object_encode:
+ * @object: a #GMimeObject
+ * @constraint: a #GMimeEncodingConstraint
+ *
+ * Calculates and sets the most efficient Content-Transfer-Encoding
+ * for this #GMimeObject and all child parts based on the @constraint
+ * provided.
+ **/
+void
+g_mime_object_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
+{
+	g_return_if_fail (GMIME_IS_OBJECT (object));
+	
+	GMIME_OBJECT_GET_CLASS (object)->encode (object, constraint);
+}
+
+
 /**
  * g_mime_object_to_string:
  * @object: a #GMimeObject
